#ifndef DATAPATHCOMPONENT_CPP
#define DATAPATHCOMPONENT_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: DataPathComponent
//
// BASE CLASSES: None.
//
// PURPOSE: Generic class describing datapath components.
//
// AUTHOR: Nicolas MOREAU
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes
#include "DataPathComponent.hpp"

//Constants

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: DataPathComponent
//
// DESCRIPTION: Constructor for DataPathComponent objects.
//
// PARAMETERS:
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
DataPathComponent::DataPathComponent(Coordinate* pCoord,
				     const int nCoord,
				     const DisplayMode eDisplayMode)
{
  m_nInput = 0;
  m_nValue = 0;
  m_pCoord = pCoord;
  m_nCoord = nCoord;
  m_eDisplayMode = eDisplayMode;
  m_eIsActivated = false;
  m_eIsUpToDate = false;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ~DataPathComponent
//
// DESCRIPTION: Destructor for DataPathComponent objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
DataPathComponent::~DataPathComponent(void)
{
  delete [] m_ppInput;
  delete [] m_pnOutput;
  m_nInput = 0;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: DependsOn
//
// DESCRIPTION: This public method adds an input to the component.
//
// PARAMETERS:
//  - nInput : The number of input the component depends on.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
DataPathComponent::DependsOn(const int nInput)
{
  if(nInput > 0)
    {
      m_ppInput = new DataPathComponent* [nInput];
      m_pnOutput = new int[nInput];
    }
 }

///////////////////////////////////////////////////////////////////////////////
//
// NAME: DependsOn
//
// DESCRIPTION: This public method adds an input to the component.
//
// PARAMETERS:
//  - pComponent : A pointer to the input component we want to add.
//  - nOutput : The input index the component depends on.
//              (Optional, defaults to 0, ie. the component's first output)
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
DataPathComponent::DependsOn(DataPathComponent* pComponent,const int nOutput)
{
  m_ppInput[m_nInput] = pComponent;
  m_pnOutput[m_nInput] = nOutput;
  m_nInput++;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: NewCycle
//
// DESCRIPTION: This public method is called each time a new cycle begins.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
DataPathComponent::NewCycle(void)
{
  m_eIsActivated = false; // The component hasn't been activated yet.
  m_eIsUpToDate = false;  // The component needs to be updated.
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Reset
//
// DESCRIPTION: This public method resets the component to its initial state.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
DataPathComponent::Reset(void)
{
  NewCycle();
  int nValue;
  for(nValue = 0 ; nValue < m_nValue ; nValue++)
    m_pwValue[nValue] = 0x00000000;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: MouseIsOver
//
// DESCRIPTION: This public method decides whether mouse is over the object.
//
// PARAMETERS: Cursor Position (Coordinate).
//
// RETURNS: A boolean: 
//  - false if the Mouse is outside the component area.
//  - true if the Mouse is inside the component area.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
bool
DataPathComponent::MouseIsOver(Coordinate* pCursor)
{
  bool eIsOver;
  if(( pCursor->X >= m_pCoord[0].X ) && ( pCursor->X <= m_pCoord[1].X ) &&
     ( pCursor->Y >= m_pCoord[0].Y ) && ( pCursor->Y <= m_pCoord[1].Y ))
    eIsOver = true;
  else
    eIsOver = false;
  return eIsOver;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: MouseAction
//
// DESCRIPTION: This public method is called when a mouse event occurs.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
DataPathComponent::MouseAction(const MouseEvent eMouseEvent)
{
  switch(eMouseEvent)
    {
    default:
    case MOUSE_CLK: // Mouse Clicked
      // Nothing to do
      break;
    }
}

#endif // DATAPATHCOMPONENT_CPP
