#ifndef LIGHTSHIFTER_CPP
#define LIGHTSHIFTER_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: LightShifter
//
// BASE CLASSES: DataPathComponent.
//
// PURPOSE: This class describes a simple LightShifter object.
//
// AUTHOR: Nicolas MOREAU & Brice DONVAL
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "LightShifter.hpp"
#include "XPM/LightShifter_disabled.xpm"
#include "XPM/LightShifter_enabled.xpm"

//Constants

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: LightShifter
//
// DESCRIPTION: Constructor for LightShifter objects.
//
// PARAMETERS:
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
LightShifter::LightShifter(Coordinate* pCoord,
			   const int nCoord,
			   const DisplayMode eDisplayMode)
  : DataPathComponent(pCoord,nCoord,eDisplayMode)
{
  m_nValue = 1;
  m_pwValue = new word[m_nValue];
  m_pwValue[0] = 0x00000000;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: ~LightShifter
//
// DESCRIPTION: Destructor for LightShifter objects.
//
// PARAMETERS: None.
//
// RETURNS: N/A.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
LightShifter::~LightShifter(void)
{
  delete [] m_pwValue;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: GetValue
//
// DESCRIPTION: This public method is used to get a value from the object.
//
// PARAMETERS:
//  - nIndex : Index of the value we want to get.
//  - eAskMode : The get mode. Legal Values are :
//           * ASK_PASV (0) : Passive mode (m_eIsActivated unchanged)
//           * ASK_ACTV (1) : Active mode (m_eIsActivated = true)
//
// RETURNS: The needed value or 0x00000000 if the value doesn't exist.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
LightShifter::GetValue(const int nIndex,const AskMode eAskMode)
{
  switch(nIndex)
    {
    case 0:
      switch(eAskMode)
	{
	case ASK_ACTV:
	  m_eIsActivated = true;
	  break;
	case ASK_PASV:	  
	default:
	  break;
	}
      m_pwValue[0] = m_ppInput[0]->GetValue(m_pnOutput[0],eAskMode) << 2;
      m_eIsUpToDate = true;
      break;
    default:
      return 0x00000000;
    }
  return m_pwValue[nIndex];
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Display
//
// DESCRIPTION: This method is used to display the component on the screen.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
LightShifter::Display(void)
{
  GdkPixmap* pPixmapTmp = NULL;
  switch(m_eDisplayMode)
    {
    default:
    case DISPLAY_GRAPHIC:
      
      if(m_eIsActivated == true)
	{
	  pPixmapTmp =
	    gdk_pixmap_create_from_xpm_d(pDrawingArea->window,NULL,NULL,
					 (gchar**)LightShifter_enabled);
	}
      else
	{
	  pPixmapTmp =
	    gdk_pixmap_create_from_xpm_d(pDrawingArea->window,NULL,NULL,
					 (gchar**)LightShifter_disabled);
	}
      gdk_draw_drawable(pPixmap,
			pDrawingArea->style->fg_gc[GTK_WIDGET_STATE(pDrawingArea)],
			pPixmapTmp,0,0,
			m_pCoord[0].X,
			m_pCoord[0].Y,
			m_pCoord[1].X - m_pCoord[0].X,
			m_pCoord[1].Y - m_pCoord[0].Y);
      gdk_pixmap_unref(pPixmapTmp);
      gtk_widget_queue_draw_area (pDrawingArea,
				  0,0,
				  pDrawingArea->allocation.width,
				  pDrawingArea->allocation.height);

      break;
    case DISPLAY_TEXT:

      break;
    }
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: MouseAction
//
// DESCRIPTION: This public method is called when a mouse event occurs.
//
// PARAMETERS: None.
//
// RETURNS: None.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
void
LightShifter::MouseAction(const MouseEvent eMouseEvent)
{
  if(m_eIsUpToDate == true)
    switch(eMouseEvent)
      {
      default:
      case MOUSE_CLK: // Mouse Clicked
	
	GtkWidget* pPopup = gtk_window_new(GTK_WINDOW_POPUP);
	gtk_window_set_position(GTK_WINDOW(pPopup),GTK_WIN_POS_MOUSE);
	gtk_window_set_resizable(GTK_WINDOW(pPopup),FALSE);
	g_signal_connect(G_OBJECT(pPopup),"destroy",
			 G_CALLBACK(gtk_widget_destroy),NULL);
	char szVal[35];
	snprintf(szVal,35,"<tt><i>Result:</i> 0x%08X</tt>",
		 (unsigned int)m_pwValue[0]);
	
	GtkWidget* pVal = gtk_label_new(NULL);
	GtkWidget* pEventBox = gtk_event_box_new();
	gtk_label_set_markup(GTK_LABEL(pVal),szVal);
	
	g_signal_connect(G_OBJECT(pPopup),"leave-notify-event",
			 G_CALLBACK(gtk_widget_destroy),NULL);
	g_signal_connect(G_OBJECT(pPopup),"button-press-event",
			 G_CALLBACK(gtk_widget_destroy),NULL);
	
	gtk_container_add(GTK_CONTAINER(pEventBox),pVal);
	gtk_container_add(GTK_CONTAINER(pPopup),pEventBox);
	gtk_widget_show_all(pPopup);
	
	break;
      }
}

#endif // LIGHTSHIFTER_CPP
