#ifndef ALU1_CPP
#define ALU1_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: ALU1
//
// BASE CLASSES: None.
//
// PURPOSE: A simple 1-bit ALU.
//
// AUTHOR: Nicolas MOREAU
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "ALU1.hpp"

//Constants

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: CarryOut
//
// DESCRIPTION: This method is used to get the carry out of the adder.
//
// PARAMETERS:
//  - a : First operand.
//  - b : Second operand.
//  - c : Carry In.
//  - bInvert : This boolean is used to know whether we need to invert b.
//
// RETURNS: The Carry Out of the adder.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
ALU1::CarryOut(byte a,byte b,byte c,byte bInvert)
{
  a=Core::Bit(a,0);                 //To be sure that only the least significant bit is taken into account
  c=Core::Bit(c,0);                 //Idem
  bInvert=Core::Bit(bInvert,0);     //Idem
  b=Core::Bit(Core::Mux(b,~b,bInvert),0); //Idem + If bInvert is set b is inverted
  return Core::Bit((a&c)|(b&c)|(a&b),0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Result
//
// DESCRIPTION: This public method is used to get the result of the ALU.
//
// PARAMETERS:
//  - a : First operand.
//  - b : Second operand.
//  - c : Carry In.
//  - l : The Less input of the ALU.
//  - op : The ALU operation to be performed. Legal operations are :
//           * AND  (0x00)
//           * OR   (0x01)
//           * ADD  (0x02)
//           * SLT  (0x03)
//           * NOR  (0x04)
//           * XOR  (0x05)
//           * ADDU (0x06)
//           * SLTU (0x07)
//  - bInvert : This boolean is used to know whether we need to invert b.
//
// RETURNS: The Result output of the ALU.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
ALU1::Result(byte a,byte b, byte c,byte l,byte op,byte bInvert)
{
  op=Core::Sel(op,2,0); //To be sure that only the three least significant bits are taken into account
  a=Core::Bit(a,0);     //To be sure that only the least significant bit is taken into account
  c=Core::Bit(c,0);             //Idem
  l=Core::Bit(l,0);             //Idem
  bInvert=Core::Bit(bInvert,0); //Idem
  b=Core::Bit(Core::Mux(b,~b,bInvert),0); //Idem + If bInvert is set b is inverted
  return Core::Bit(Core::Mux(a&b,a|b,(a&~b&~c)|(~a&b&~c)|(~a&~b&c)|(a&b&c),l,~(a|b),a^b,(a&~b&~c)|(~a&b&~c)|(~a&~b&c)|(a&b&c),l,op),0);
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Set
//
// DESCRIPTION: This public method is used to get the Set output of the ALU.
//
// PARAMETERS:
//  - a : First operand.
//  - b : Second operand.
//  - c : Carry In.
//  - op : The ALU operation to be performed. Legal operations are :
//           * AND  (0x00)
//           * OR   (0x01)
//           * ADD  (0x02)
//           * SLT  (0x03)
//           * NOR  (0x04)
//           * XOR  (0x05)
//           * ADDU (0x06)
//           * SLTU (0x07)
//  - bInvert : This boolean is used to know whether we need to invert b.
//
// RETURNS: The Set output of the ALU.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
ALU1::Set(byte a,byte b,byte c,byte op,byte bInvert)
{
  op=Core::Bit(op,2); //To be sure that only the bit #2 is taken into account
  a=Core::Bit(a,0);   //To be sure that only the least significant bit is taken into account
  c=Core::Bit(c,0);             //Idem
  bInvert=Core::Bit(bInvert,0); //Idem
  b=Core::Bit(Core::Mux(b,~b,bInvert),0); //Idem + If bInvert is set b is inverted
  return Core::Bit(Core::Mux((a&~b&~c)|(~a&b&~c)|(~a&~b&c)|(a&b&c),~((a&c)|(b&c)|(a&b)),op),0);
}

#endif // ALU1_CPP
