#ifndef CORE_CPP
#define CORE_CPP

///////////////////////////////////////////////////////////////////////////////
//
// $Header: $
//
// NAME: Core
//
// BASE CLASSES: None.
//
// PURPOSE: Core functions.
//
// AUTHOR: Nicolas MOREAU
//
//  COPYRIGHT NOTICE:
//
// The KOYOTE Project
// Copyright (C) 2003-2004 Nicolas MOREAU & Brice DONVAL
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  REVISION HISTORY:
// $Log: $
//
///////////////////////////////////////////////////////////////////////////////

//Includes

#include "Core.hpp"

//Constants

word START_ADDR;
word END_ADDR;

//Externals

//Types and classes

//Member Functions

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Bit
//
// DESCRIPTION: This function selects a bit in a word.
//
// PARAMETERS:
//  - w : The word we want to extract the bit from.
//  - b : The index of the bit we need. Index range is : [31..0].
//
// RETURNS: The bit needed.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
byte
Core::Bit(word w,byte b)
{
  b=b&0x1F; // To be sure that b is in the index range
  return (w>>b)&0x01;
}
///////////////////////////////////////////////////////////////////////////////
//
// NAME: Sel
//
// DESCRIPTION: This function is used to get a bit subset of a word.
//
// PARAMETERS:
//  - w : The word we want to extract the bit from.
//  - e : End index of the subset we need. Index range is : [31..0].
//  - s : Start index of the subset we need. Index range is : [31..0].
//
// RETURNS: The subset needed. The selected bits are in result bits [(e-s)..0].
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
Core::Sel(word w,byte e,byte s)
{
  e=e&0x1F; //To be sure that e is in the index range
  s=s&0x1F; //Idem for s
  word r=0x00000000;
  byte b;
  for(b=0;b<=(e-s);b++)
    r=r|(Bit(w,s+b)<<b);
  return r;
}
///////////////////////////////////////////////////////////////////////////////
//
// NAME: Sub
//
// DESCRIPTION: This function is used to get a bit subset of a word.
//
// PARAMETERS:
//  - w : The word we want to extract the bit from.
//  - e : End index of the subset we need. Index range is : [31..0].
//  - s : Start index of the subset we need. Index range is : [31..0].
//
// RETURNS: The subset needed. The selected bits are in result bits [e..s].
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
Core::Sub(word w,byte e,byte s)
{
  return Sel(w,e,s)<<s;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Mux
//
// DESCRIPTION: A simple 2-ways 32-bits multiplexor.
//
// PARAMETERS:
//  - w1 : First input.
//  - w2 : Second input.
//  - muxControl : The multiplexor control line.
//
// RETURNS: The selected input.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
Core::Mux(word w1,word w2,byte muxControl)
{
  return (Bit(muxControl,0)==0x00)?w1:w2;
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Mux
//
// DESCRIPTION: A simple 3-ways 32-bits multiplexor.
//
// PARAMETERS:
//  - w1 : First input.
//  - w2 : Second input.
//  - w3 : Third input.
//  - muxControl : The multiplexor control line.
//
// RETURNS: The selected input.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
Core::Mux(word w1,word w2,word w3,byte muxControl)
{
  return Core::Mux(Core::Mux(w1,w3,Core::Bit(muxControl,1)),w2,Core::Bit(muxControl,0));
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Mux
//
// DESCRIPTION: A simple 4-ways 32-bits multiplexor
//
// PARAMETERS:
//  - w1 : First input.
//  - w2 : Second input.
//  - w3 : Third input.
//  - w4 : Fourth input.
//  - muxControl : The multiplexor control lines.
//
// RETURNS: The selected input.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
Core::Mux(word w1,word w2,word w3,word w4,byte muxControl)
{
  return Mux(Mux(w1,w3,Bit(muxControl,1)),Mux(w2,w4,Bit(muxControl,1)),Bit(muxControl,0));
}
///////////////////////////////////////////////////////////////////////////////
//
// NAME: Mux
//
// DESCRIPTION: A simple 8-ways 32-bits multiplexor
//
// PARAMETERS:
//  - w1 : First input.
//  - w2 : Second input.
//  - w3 : Third input.
//  - w4 : Fourth input.
//  - w5 : Fifth input.
//  - w6 : Sixth input.
//  - w7 : Seventh input.
//  - w8 : Heighth input.
//  - muxControl : The multiplexor control lines.
//
// RETURNS: The selected input.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
Core::Mux(word w1,word w2,word w3,word w4,word w5,word w6,word w7,word w8,byte muxControl)
{
  return Mux(Mux(w1,w2,w3,w4,Sel(muxControl,1,0)),Mux(w5,w6,w7,w8,Sel(muxControl,1,0)),Bit(muxControl,2));
}

///////////////////////////////////////////////////////////////////////////////
//
// NAME: Ext
//
// DESCRIPTION: This function sign-extends a 16-bits halfword
//
// PARAMETERS:
//  - w : The halfword that needs to be sign-extended.
//
// RETURNS: The result of the sign-extension.
//
// SIDE EFFECTS: None.
//
///////////////////////////////////////////////////////////////////////////////
word
Core::Ext(word w)
{
  return Mux(w&0x0000FFFF,w|0xFFFF0000,Bit(w,15));
}

#endif // CORE_CPP
